﻿/******************************************************************************
	Afterimage CPU & Memory meter
******************************************************************************/

/*****************************
	変数宣言
*****************************/
var oBackground;												/* 背景オブジェクト */
var oImg_lable;													/* 表示ラベルオブジェクト */
var oImg_unit;													/* 単位ラベルオブジェクト */
var oImg_100;													/* 数値オブジェクト(百の位) */
var oImg_10;													/* 数値オブジェクト(十の位) */
var oImg_1;														/* 数値オブジェクト(一の位) */
var oImg_dot;													/* 小数点オブジェクト */
var oImg_bar;													/* バーオブジェクト配列 */
var oRefleshTimer;												/* 更新タイマ管理オブジェクト */

var cpu_num;													/* CPU数 */

var num_interval = 0;											/* 数値表示用インターバル */
var now_bar;													/* バー現在値 */
var total_delta;												/* 積分項 */
var meter_sel = 0;												/* 表示メータ選択 (0:CPU 1:Memory) */

var number_pre = 0;												/* 前回数値 */
var decimal_pre = 0;											/* 前回小数点以下値 */
var int_bar_pre;												/* 前回バー位置 */
var afterimage = true;											/* 残像表示設定 */
var bar_opacity_dec = 20;										/* バー不透明度減衰量 */
var bar_min_opacity = 20;										/* バー不透明度最小値 */

/*****************************
	定数宣言
*****************************/
var BG_WIDTH = 130;												/* 背景の横幅 */
var BG_HEIGHT = 43;												/* 背景の縦幅 */
var NUM_BASE_X = 38;											/* 数字の基本横位置 */
var NUM_BASE_Y = 1;												/* 数字の基本縦位置 */
var NUM_WIDTH = 21;												/* 数字の横幅 */
var BAR_BASE_X = 0;												/* バー表示の基本横位置 */
var BAR_BASE_Y = 32;											/* バー表示の基本縦位置 */
var BG_OPACITY = 0;												/* 背景の不透明度 */
var REFLESH_INTERVAL = 100;										/* 表示更新間隔(ms) */
var CPU_NUM_INTERVAL = 700;										/* CPU使用率 数値更新間隔(ms) */
var GAIN_P = 0.5;												/* 係数P: 比例ゲイン */
var GAIN_I = 0.1;												/* 係数I: 積分ゲイン */
var SEL_CPU = 0;												/* CPU選択値 */
var SEL_MEM = 1;												/* メモリ選択値 */

/*****************************
	設定画面＆ハンドラ設定
*****************************/
System.Gadget.settingsUI = "settings.html";						/* 設定画面指定 */
System.Gadget.onSettingsClosed = applySettings;					/* 設定完了時ハンドラ指定 */


/************************************************************************
	Function	: initGadget
	Description	: ガジェット初回起動時の初期化を行う。
	Input		: 
	Output		:
	Data		: 2008/08/12
	Copyright	: 2008 KOJIMA Shuichi
************************************************************************/
function initGadget(){
	
	/*-------------------
		配列生成
	-------------------*/
	cpu_num = System.Machine.CPUs.count;						/* CPUスレッド数取得 */
	now_bar = new Array(cpu_num);								/* バー現在値の配列宣言 */
	total_delta = new Array(cpu_num);							/* 積分項の配列宣言 */
	int_bar_pre = new Array(cpu_num);							/* 前回バー位置の配列宣言 */
	oImg_bar = new Array(cpu_num);								/* スレッド数で配列宣言 */
	for (var i=0; i<cpu_num; i++){								/* 配列宣言用ループ */
		oImg_bar[i] = new Array(25);							/* バー配列宣言 */
		now_bar[i] = 0;
		total_delta[i] = 0;
		int_bar_pre[i] = 0;
	}
	
	/*-------------------
		バー初期設定
	-------------------*/
	bar_min_opacity = System.Gadget.Settings.readString("OpacitySel");	/* バー最低透明度を読み出し */
	if (bar_min_opacity == ""){									/* バー最低透明度が未設定か？ */
		bar_min_opacity = 20;									/* 最低透明度を20％に設定 */
	}
	bar_opacity_dec = (100 - bar_min_opacity) / 4;				/* バー不透明度減衰量を算出 */
	
	/*-------------------
		背景設定
	-------------------*/
	document.body.style.width = BG_WIDTH + "px";				/* 横幅設定 */
	document.body.style.height = BG_HEIGHT + (11 * (cpu_num - 1)) + "px";	/* 縦幅設定 */
	oBackground = document.getElementById("background");		/* 背景オブジェクトを取得 */
	oBackground.style.width = BG_WIDTH + "px";					/* 横幅設定 */
	oBackground.style.height = BG_HEIGHT + (11 * (cpu_num - 1)) + "px";	/* 縦幅設定 */
	oBackground.opacity = BG_OPACITY;							/* 不透明度設定 */
	
	/*-------------------
		画像設定
	-------------------*/
	oBackground.addImageObject("url(images/line.png)", 0, 30);	/* 区切り線追加 */
	oImg_label = oBackground.addImageObject("url(images/label_cpu.png)", 1, 15);	/* CPUラベル追加 */
	oImg_unit  = oBackground.addImageObject("url(images/par.png)", 101, 13);	/* パーセント記号追加 */
	oImg_dot = oBackground.addImageObject("url(images/null.png)", NUM_BASE_X + NUM_WIDTH * 1.5, NUM_BASE_Y);	/* 小数点追加 */
	oImg_100 = oBackground.addImageObject("url(images/null.png)", NUM_BASE_X + NUM_WIDTH * 0, NUM_BASE_Y);		/* 百の位追加 */
	oImg_10  = oBackground.addImageObject("url(images/null.png)", NUM_BASE_X + NUM_WIDTH * 1, NUM_BASE_Y);		/* 十の位追加 */
	oImg_1   = oBackground.addImageObject("url(images/null.png)", NUM_BASE_X + NUM_WIDTH * 2, NUM_BASE_Y);		/* 一の位追加 */
	for (var j=0; j<cpu_num; j++){								/* バー表示用ループ */
		for (var i=0; i<25; i++){								/* バー表示用ループ */
			oImg_bar[j][i] = oBackground.addImageObject("url(images/bar.png)", i*4 +  BAR_BASE_X, j*11 + BAR_BASE_Y); /* バー表示 */
			oImg_bar[j][i].opacity = bar_min_opacity;			/* バー初期状態は最低透明度に設定 */
		}
	}

	/*-------------------
		動作開始
	-------------------*/
	meter_sel = System.Gadget.Settings.readString("MeterSel");	/* 表示メータ選択取得 */
	if (meter_sel == ""){										/* 表示メータ選択が未設定か？ */
		meter_sel = SEL_CPU;									/* CPU選択 */
	}

	if (meter_sel == SEL_CPU){									/* CPU選択か？ */
		document.body.style.height = BG_HEIGHT + (11 * (cpu_num - 1)) + "px";	/* CPU表示用の縦幅設定 */
		oBackground.style.height = BG_HEIGHT + (11 * (cpu_num - 1)) + "px";	/* 縦幅設定 */
	}
	else {														/* メモリ選択の場合 */
		document.body.style.height = BG_HEIGHT + "px";			/* メモリ表示用の縦幅設定 */
		oBackground.style.height = BG_HEIGHT + "px";			/* 縦幅設定 */
	}

	num_interval = 0;											/* 表示更新タイマ初期化 */
	refleshDisp();												/* 数値表示更新開始 */
}

/************************************************************************
	Function	: applySettings
	Description	: ガジェット初回起動時の初期化を行う。
	Input		: event: イベントオブジェクト
	Output		:
	Data		: 2008/08/13
	Copyright	: 2008 KOJIMA Shuichi
************************************************************************/
function applySettings(event){
	if (event.closeAction == event.Action.commit){				/* 設定ダイアログでOKボタンが押されたか？ */
		clearTimeout(oRefleshTimer);							/* 更新タイマをいったんキャンセル */
		meter_sel = System.Gadget.Settings.readString("MeterSel");	/* 表示メータ選択取得 */
		if (meter_sel == SEL_CPU){								/* CPU選択か？ */
			document.body.style.height = BG_HEIGHT + (11 * (cpu_num - 1)) + "px";	/* CPU表示用の縦幅設定 */
			oBackground.style.height = BG_HEIGHT + (11 * (cpu_num - 1)) + "px";	/* 縦幅設定 */
		}
		else {													/* メモリ選択の場合 */
			document.body.style.height = BG_HEIGHT + "px";		/* メモリ表示用の縦幅設定 */
			oBackground.style.height = BG_HEIGHT + "px";		/* 縦幅設定 */
		}
		
		bar_min_opacity = System.Gadget.Settings.readString("OpacitySel");	/* バー最低透明度を読み出し */
		for (var j=0; j<cpu_num; j++){							/* バー表示用ループ */
			for (var i=0; i<25; i++){							/* バー表示用ループ */
				oImg_bar[j][i].opacity = bar_min_opacity;		/* バー最低透明度を更新 */
			}
		}
		
		for (var i=0; i<cpu_num; i++){							/* バー表示初期化用ループ */
			now_bar[i] = 0;										/* バー表示関係パラメータを初期化する */
			total_delta[i] = 0;
			int_bar_pre[i] = 0;
		}

		num_interval = 0;										/* 表示更新タイマ初期化 */
		refleshDisp();											/* 数値表示更新 */
	}
}

/************************************************************************
	Function	: 
	Description	: 数値表示の表示更新を行う。
	Input		: 
	Output		:
	Data		: 2008/08/13
	Copyright	: 2008 KOJIMA Shuichi
************************************************************************/
function refleshDisp(){
	
	if (meter_sel == SEL_CPU){									/* CPU選択か？ */
		if (num_interval == 0){									/* 表示タイマタイムアウトか？ */
			oCpuStatus = new cpuStatus();						/* CPU使用率オブジェクトを取得 */
			dispNumber(oCpuStatus.cpuUsage);					/* CPU使用率を表示 */
		}
		for (var i=0; i<cpu_num; i++){							/* バー表示用ループ */
			dispBar(oCpuStatus.threadUsage[i] / 4, i);			/* バー表示 */
		}
	}
	else{														/* メモリ選択の場合 */
		if (num_interval == 0){									/* 表示タイマタイムアウトか？ */
			oMemStatus = new memStatus();						/* メモリ使用量オブジェクトを取得 */
			dispNumber(oMemStatus.usedMemory);					/* メモリ使用量を表示 */
		}
		dispBar(oMemStatus.memUsage / 4, 0);					/* バー表示 */
	}
	
	num_interval++;
	num_interval %= Math.floor(CPU_NUM_INTERVAL / REFLESH_INTERVAL);

	oRefleshTimer = setTimeout("refleshDisp()", REFLESH_INTERVAL);	/* 表示更新タイマ再設定 */
}

/************************************************************************
	Function	: dispNumber
	Description	: 数値表示を行う。
	Input		: num: 表示する数値 (正数を指定)
	Output		:
	Data		: 2008/08/12
	Copyright	: 2008 KOJIMA Shuichi
************************************************************************/
function dispNumber(num){

	if (num >= 1000){											/* 表示値が4桁以上になるか？ */
		number = Math.floor(num / 1000);						/* 基数を1kに正規化 */
		decimal = Math.floor(num / 100) % 10;					/* 小数点以下の値を取得 */
	}
	else{
		number = Math.floor(num);								/* 表示値の整数値を取得 */
		decimal = Math.round(num * 10) % 10;					/* 小数点以下の値を取得 */
	}
	
	if ((number != number_pre) || (decimal != decimal_pre)){	/* 表示数値に変化ありか？ */
		number_pre = number;									/* 今回の表示値を記憶 */
		decimal_pre = decimal;									/* 今回の小数点以下値を記憶 */
		
		number_100 = Math.floor(number / 100);						/* 百の位の値を取得 */
		number_10  = Math.floor(Math.floor(number / 10) % 10);		/* 十の位の値を取得 */
		number_1   = Math.floor(number % 10);						/* 一の位の値を取得 */
	
		if (decimal > 0){											/* 小数点以下の値が存在するか？ */
			number_100 = number_10;									/* 一桁分左シフトして小数点以下第一位まで表示 */
			number_10 = number_1;
			number_1 = decimal;
			oImg_dot.src = "url(images/dot.png)";					/* 小数点表示 */
		}
		else{														/* 値が整数の場合 */
			oImg_dot.src = "url(images/null.png)";					/* 小数点表示なし */
		}
	
		if (number_100 > 0){										/* 最上位に値が存在するか？ */
			oImg_100.src = "url(images/"+ number_100 +".png)";		/* 最上位の値を表示 */
		}
		else{														/* 最上位に値が存在しない(ゼロ)の場合 */
			oImg_100.src = "url(images/null.png)";					/* 最上位は空白 */
		}
	
		if ((number_10 > 0)												/* 第二位に値が存在するか？ */
		 || ((number_10 == 0) && ((decimal > 0) || (number_100 > 0)))){	/* or 第二位がゼロだが、小数点以下に値が存在or三桁表示か？ */
			oImg_10.src  = "url(images/"+ number_10 +".png)";		/* 第二位の数値表示 */
		}
		else{														/* それ以外の場合 */
			oImg_10.src  = "url(images/null.png)";					/* 第二位は空白 */
		}
	
		oImg_1.src   = "url(images/"+ number_1 +".png)";			/* 最下位の値を表示 */
	
		if (meter_sel == SEL_CPU){										/* CPU表示中か？ */
			oImg_label.src = "url(images/label_cpu.png)";			/* CPUラベル表示 */
			oImg_unit.src = "url(images/par.png)";					/* パーセント記号表示 */
		}
		else{														/* それ以外の場合 */
			oImg_label.src = "url(images/label_mem.png)";			/* メモリラベル表示 */
			if (num >= 1000){										/* 表示桁数が4桁以上か？ */
				oImg_unit.src = "url(images/GB.png)";				/* 単位GB表示 */
			}
			else{													/* 表示桁数が4桁以上の場合 */
				oImg_unit.src = "url(images/MB.png)";				/* 単位MB表示 */
			}
		}
	}
}

/************************************************************************
	Function	: dispBar
	Description	: バー表示を行う。
	Input		: num: 表示基準値(25で正規化のこと)
	Output		:
	Data		: 2008/08/12
	Copyright	: 2008 KOJIMA Shuichi
************************************************************************/
function dispBar(num, index){
	delta = num - now_bar[index];								/* 目標値との差分を算出 */
	total_delta[index] += GAIN_I * delta;						/* 積分項の算出 */
	now_bar[index] += GAIN_P * delta + total_delta[index];		/* 現在値更新 */
	
	int_bar = Math.max(Math.min(Math.round(now_bar[index]), 25), 0);	/* 表示用の整数値を取得 */
	if (int_bar != int_bar_pre[index]){							/* 表示値が前回値と異なる？ */
		for (var i=0; i<int_bar; i++){							/* バー表示用ループ */
			oImg_bar[index][i].opacity = 100;					/* 表示用のバーを不透明化 */
		}
		int_bar_pre[index] = int_bar;							/* 今回のバー位置情報を記憶 */
	}
	
	for (var j=int_bar; j<25; j++){								/* バー残像用のループ */
		if (oImg_bar[index][j].opacity > bar_min_opacity){		/* バーの不透明度が残っているか？ */
			oImg_bar[index][j].opacity = Math.max(oImg_bar[index][j].opacity - bar_opacity_dec , bar_min_opacity);	/* 不透明度を減衰 */
		}
		else { 													/* すでに透明の領域の場合 */
			break;												/* ループを抜ける */
		}
	}
}

/************************************************************************
	Function	: cpuStatus
	Description	: CPUの状態を取得する。
	Input		: 
	Output		: 
	Data		: 2008/08/12
	Copyright	: 2008 KOJIMA Shuichi
************************************************************************/
function cpuStatus(){
	this.cpuCount = System.Machine.CPUs.count;					/* CPUコア数を取得 */
	this.threadUsage  = new Array(this.cpuCount);				/* スレッド使用率の配列 */
	
	cpuUsageSum = 0;
	for (var i=0; i<this.cpuCount; i++){						/* CPU使用率を取得するためのループ */
		this.threadUsage[i] = System.Machine.CPUs.item(i).usagePercentage;	/* スレッドあたりの使用率を取得 */
		cpuUsageSum += this.threadUsage[i];						/* CPU使用率の累計を取得 */
		this.threadUsage[i] = Math.floor(this.threadUsage[i]);	/* スレッドあたりの使用率を整数値化 */
	}
	
	this.cpuUsage = Math.floor(cpuUsageSum / this.cpuCount);	/* CPU使用率を取得 */
}

/************************************************************************
	Function	: memStatus
	Description	: メモリの状態を取得する。
	Input		: 
	Output		: 
	Data		: 2008/08/13
	Copyright	: 2008 KOJIMA Shuichi
************************************************************************/
function memStatus(){
    this.totalMemory = System.Machine.totalMemory;				/* 総メモリ量取得 */
    this.availableMemory = System.Machine.availableMemory;		/* 利用可能メモリ量取得 */
    this.usedMemory = this.totalMemory - this.availableMemory;	/* 使用メモリ量取得 */

	if((this.totalMemory > 0) && (this.totalMemory > this.availableMemory)){
		this.memUsage = Math.min(100 - (this.availableMemory / this.totalMemory * 100), 100);
	}
	else{
		this.memUsage = 0;
	}
}